"""World Weather Clock."""
import board
from adafruit_ble import BLERadio
from adafruit_ble.advertising.standard import ProvideServicesAdvertisement
from adafruit_ble.services.nordic import UARTService
from adafruit_st7735r import ST7735R
from adafruit_bitmap_font import bitmap_font
from adafruit_display_text import label
from adafruit_display_shapes.roundrect import RoundRect
from adafruit_esp32spi import adafruit_esp32spi
from adafruit_esp32spi import adafruit_esp32spi_wifimanager
from digitalio import DigitalInOut
import displayio
import rtc
from secrets import secrets
from terminalio import FONT, Terminal
from time import localtime
import weather

# Initialize real time clock (time will be set using API)
clock = rtc.RTC()

ble = BLERadio()
uart_server = UARTService()
advertisement = ProvideServicesAdvertisement(uart_server)

spi = board.SPI()

esp32_cs = DigitalInOut(board.D10)
esp32_ready = DigitalInOut(board.D11)
esp32_reset = DigitalInOut(board.D12)

esp = adafruit_esp32spi.ESP_SPIcontrol(spi, esp32_cs, esp32_ready, esp32_reset)
wifi = adafruit_esp32spi_wifimanager.ESPSPI_WiFiManager(esp, secrets, None)

displayio.release_displays()
display_cs = board.D9
display_dc = board.D6
display_reset = board.D5
display_bus = displayio.FourWire(spi, command=display_dc,
                                 chip_select=display_cs,
                                 reset=display_reset)
display = ST7735R(display_bus, width=128, height=160)
splash = displayio.Group(max_size=7)
display.show(splash)

# Background color
background_bitmap = displayio.Bitmap(128, 160, 1)
palette = displayio.Palette(2)
palette[0] = 0xFFFF00
palette[1] = 0x0
background = displayio.TileGrid(background_bitmap, x=0, y=0,
                                pixel_shader=palette)
splash.append(background)

# Load BDF font
font = bitmap_font.load_font('lode.bdf')
# Preload glyphs, otherwise very slow
font.load_glyphs(b' %-./0123456789:ABCDEFGHIJKLMNOPQRSTUVWXYZ\xb0')

# City Label
text_city = label.Label(font, max_glyphs=13, x=8, y=6, color=0xFF0000)
splash.append(text_city)

# Clock outline
rect = RoundRect(0, 15, 128, 25, 10, fill=0x0, outline=0x00ff00, stroke=3)
splash.append(rect)

# Clock Label
text_clock = label.Label(font, max_glyphs=14,
                         x=8, y=27, color=0xFFFFFF)
splash.append(text_clock)

# Condition Terminal
w, h = FONT.get_bounding_box()
condition = displayio.TileGrid(FONT.bitmap, pixel_shader=palette,
                               x=6, y=40, width=17, height=5,
                               tile_width=w, tile_height=h)

terminal = Terminal(condition, FONT)
splash.append(condition)
terminal.write('Awaiting\r\nBluetooth\r\nCommand.\r\n\r\n')

# Temp / Humidity Label
text_temp = label.Label(font, max_glyphs=14, x=12, y=136, color=0x0000FF)
splash.append(text_temp)

with open("/icons.bmp", "rb") as f:
    bmp = displayio.OnDiskBitmap(f)
    icons = displayio.TileGrid(bmp,
                               pixel_shader=displayio.ColorConverter(),
                               default_tile=12,
                               x=77, y=110,
                               width=1, height=1,
                               tile_width=50, tile_height=50)
    splash.append(icons)
    clock_ready = False
    ble.start_advertising(advertisement) 

    # Wait for a connection
    while not ble.connected:
        pass

    while ble.connected:
        if uart_server.in_waiting:
            city = uart_server.read().decode('utf-8').rstrip().upper()
            if city in weather.cities.keys():
                text_city.text = ''
                # Clear display
                text_clock.text = ''
                clock_ready = False
                previous_second = 0
                text_temp.text = ''
                icons[0] = 12
                terminal.write('\r\nPlease Wait.\r\n\r\n\r\n\r\n')
                text_city.text = city
                # Call API
                data = weather.getWeatherConditions(city, wifi)
                # Set real time clock
                clock.datetime = data['datetime']
                text_clock.text = '{0:02}:{1:02}:{2:02} {3:02}/{4:02}'.format(
                    localtime().tm_hour,
                    localtime().tm_min,
                    localtime().tm_sec,
                    localtime().tm_mon,
                    localtime().tm_mday)
                clock_ready = True
                terminal.write('\r\n{0}\r\nSunset  {1}\r\nSunrise {2}\r\nWind \
{3:.1f}KPH'.format(data['condition'],
                   data['sunset'],
                   data['sunrise'],
                   data['wind']))
                # Align terminal text to top
                if len(data['condition']) <= 16:
                    terminal.write('\r\n')
                # Display temperature and humidity
                text_temp.text = '{0:.1f}\xb0C\n{1}%'.format(
                    data['temperature'],
                    data['humidity'])
                # Set condition icon
                icons[0] = data['icon_frame']
            else:
                uart_server.write('City Not Found.')
        else:
            if clock_ready:
                lt = localtime()
                # Update clock if seconds have changed
                if lt.tm_sec != previous_second:
                    previous_second = lt.tm_sec
                    text_clock.text = '{0:02}:{1:02}:{2:02} {3:02}/{4:02}'.\
                        format(
                            lt.tm_hour,
                            lt.tm_min,
                            lt.tm_sec,
                            lt.tm_mon,
                            lt.tm_mday)